"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const literals_1 = require("./literals");
/**
 * The startegy for console.log calls.
 *
 * @export
 * @class ConsoleLogStrategy
 * @implements {ParserStrategy}
 */
class ConsoleLogStrategy {
    constructor() {
        /**
         * The strategies need by this strategy.
         *
         * @private
         * @type {ParserStrategy[]}
         * @memberof ConsoleLogStrategy
         */
        this.strategies = [
            new literals_1.StringLiteralStrategy(),
            new literals_1.NumberLiteralStrategy(),
            new literals_1.BooleanLiteralStrategy(),
        ];
    }
    /**
     * Checks whether this strategy supports the given `name`.
     *
     * @param {Token} token The name to check for.
     * @returns {boolean} Whether this strategy supports the given `name`.
     * @memberof ConsoleLogStrategy
     */
    supports(token) {
        if (!token || token === null) {
            return false;
        }
        return token.value === 'print';
    }
    /**
     * Handle the `token`.
     *
     * @param {Token|Token[]} token The token to handle.
     * @param {number} [index] The index to start at.
     * @returns {ParserStrategyModel} The expression or the new token made from the `token`.
     * @throws {TypeError} Thrown when an error occurred while creating the expression.
     * @throws {SyntaxError} Thrown when the syntax is incorrect.
     * @throws {Error} Thrown when no index was provided.
     * @memberof ConsoleLogStrategy
     */
    handle(token, index) {
        const fragment = { type: 'ConsoleLogCall', params: [] };
        if (index === undefined) {
            throw new Error('No index provided.');
        }
        if (!Array.isArray(token)) {
            throw new Error('Non-Array provided.');
        }
        index++;
        if (token[index].type === 'openingRoundParen') {
            index++;
            for (let i = index; i < token.length; i++) {
                if (token[i].type === 'closingRoundParen') {
                    break;
                }
                const strategy = this.strategies.find(s => s.supports(token[i]));
                if (strategy) {
                    fragment.params.push(strategy.handle(token[i])
                        .tokenOrExpression);
                }
                else {
                    throw new TypeError('Trying to print unknown type.');
                }
                index++;
            }
            if (index === token.length) {
                throw new SyntaxError('Invalid call. Closing round parenthesis is missing.');
            }
        }
        else {
            throw new SyntaxError('Invalid call. Opening round parenthesis is missing.');
        }
        return { tokenOrExpression: fragment, position: index };
    }
}
exports.ConsoleLogStrategy = ConsoleLogStrategy;
//# sourceMappingURL=console-log-strategy.js.map